@@ -313,8 +313,8 @@ program SEQ
   &                   +dgammadr(i,j)*N2(i,j)
         end if
 
-        force(i,j)=r(i)*rho(i,j)*(dQdr(i,j)+gamma(i,j)*dQdz(i,j))
-!        force(i,j)=r(i)*rho(i,j)*(dQdr(i,j)-dximomdz(i,j)+gamma(i,j)*dQdz(i,j))
+!        force(i,j)=r(i)*rho(i,j)*(dQdr(i,j)+gamma(i,j)*dQdz(i,j))
+        force(i,j)=r(i)*rho(i,j)*(dQdr(i,j)-dximomdz(i,j)+gamma(i,j)*dQdz(i,j))
 !write(*,*) term1(i,j), term2(i,j), term3(i,j), term4(i,j), term5(i,j), force(i,j), i, j
      end do
   end do
@@ -391,6 +391,7 @@ end do
 
   call HistoryClose
 
+write(*,*) "mom", mom
 !-- λ
 
   write(*,*) "Program is terminated, normally."
