@@ -12,8 +12,8 @@ program cloud_1d
   implicit none
 
   integer :: i, j, k, nmax, nt, dmpstp
-  real, parameter :: zmax=5000.0   ! vertical wind maximum height.
-  real, parameter :: wfact=2.0   ! vertical wind maximum height.
+  real, parameter :: zmax=2500.0   ! vertical wind maximum height.
+  real, parameter :: wfact=0.5   ! vertical wind maximum height.
   real :: dt, rho0
   real :: VDvr_term, NUAvi_term, VDvi_term, VDvs_term, VDvg_term
   real :: NUFci_term, NUCci_term, NUHci_term, CLcs_term, CLcg_term
@@ -22,8 +22,9 @@ program cloud_1d
   real :: MLic_term, MLsr_term, MLgr_term
   real :: FRrg_term, SHsr_term, SHgr_term
   real :: CNcr_term, CNis_term, CNsg_term
-  real :: SPsi_term, SPgi_term, PG_term
-  real :: ars
+  real :: SPsi_term, SPgi_term, PGd_term, PGw_term, CLrsg_term
+  real :: VENTr_term, VENTs_term, VENTg_term, Gw_term, Gi_term
+  real :: ars, coe1, dtb
   real, allocatable, dimension(:) :: z, w, qv, qr, qc, rho, pt
   real, allocatable, dimension(:) :: qi, qs, qg
   real, allocatable, dimension(:) :: old_qv, old_qr, old_qc, old_pt
@@ -37,6 +38,7 @@ program cloud_1d
   real, allocatable, dimension(:) :: forceqv, forceqc, forceqr, forcept
   real, allocatable, dimension(:) :: forceqi, forceqs, forceqg
   real, dimension(2) :: qvbc, qcbc, qrbc, qibc, qsbc, qgbc, ptbc
+  real, dimension(3) :: tmptend
   character(100) :: fname, outname
   character(20), allocatable, dimension(:,:) :: cval
 
@@ -133,6 +135,7 @@ program cloud_1d
   qibc=0.0
   qsbc=0.0
   qgbc=0.0
+  CLrsg_term=0.0
 
 !-- output file is initialized.
 
@@ -172,6 +175,12 @@ program cloud_1d
 
   do i=1,nt
 
+     if(i==1)then  ! first step is Euler method.
+        dtb=dt
+     else
+        dtb=2.0*dt
+     end if
+
   !-- initializing forcing terms
 
      do j=1,nmax
@@ -196,97 +205,10 @@ program cloud_1d
         fallqg(j)=rho(j)*UhxN( 'g', qg(j), rho(j), rho0 )*qg(j)
      end do
 
-  !-- calculating microphysics schemes
-
-     do j=2,nmax-1
-        if(qv(j)>0.0.or.qc(j)>0.0.or.qr(j)>0.0)then
-           NUAvi_term=NUAvi( temp(j), pres(j), qv(j), qi(j), rho(j) )
-           NUFci_term=NUFci( temp(j), qc(j), rho(j) )
-           NUCci_term=NUCci( temp(j), pres(j), qc(j), rho(j) )
-           NUHci_term=NUHci( temp(j), qc(j), dt )
-           VDvr_term=VDvx( 'r', temp(j), pres(j), qv(j), qc(j), qr(j),  &
-  &                        qr(j), rho(j), rho0 )
-           VDvi_term=VDvi( temp(j), pres(j), qv(j), qi(j), rho(j) )
-           VDvs_term=VDvx( 's', temp(j), pres(j), qv(j), qc(j), qr(j),  &
-  &                        qs(j), rho(j), rho0 )
-           VDvg_term=VDvx( 'g', temp(j), pres(j), qv(j), qc(j), qr(j),  &
-  &                        qg(j), rho(j), rho0 )
-           CLcs_term=CLxy( 'cs', temp(j), pres(j), qc(j), qs(j), rho(j), rho0 )
-           CLcg_term=CLxy( 'cg', temp(j), pres(j), qc(j), qg(j), rho(j), rho0 )
-           CLri_term=CLxy( 'ri', temp(j), pres(j), qr(j), qi(j), rho(j), rho0, qv=qv(j) )
-           CLrs_term=CLxy( 'rs', temp(j), pres(j), qr(j), qs(j), rho(j), rho0 )
-           CLrg_term=CLxy( 'rg', temp(j), pres(j), qr(j), qg(j), rho(j), rho0 )
-           CLcr_term=CLxy( 'cr', temp(j), pres(j), qc(j), qr(j), rho(j), rho0 )
-           CLir_term=CLxy( 'ir', temp(j), pres(j), qi(j), qr(j), rho(j), rho0 )
-           CLis_term=CLxy( 'is', temp(j), pres(j), qi(j), qs(j), rho(j), rho0 )
-           CLig_term=CLxy( 'ig', temp(j), pres(j), qi(j), qg(j), rho(j), rho0 )
-           CLsr_term=CLxy( 'sr', temp(j), pres(j), qs(j), qr(j), rho(j), rho0 )
-           CLsg_term=CLxy( 'sg', temp(j), pres(j), qs(j), qg(j), rho(j), rho0 )
-           MLic_term=MLic( temp(j), qi(j), dt )
-           MLsr_term=MLxr( 's', temp(j), pres(j), qv(j), qc(j), qr(j),  &
-  &                        qs(j), rho(j), rho0 )
-           MLgr_term=MLxr( 'g', temp(j), pres(j), qv(j), qc(j), qr(j),  &
-  &                        qg(j), rho(j), rho0 )
-           PG_term=PGdw( temp(j), pres(j), qv(j), qc(j), qr(j), qi(j),  &
-  &                      qs(j), qg(j), rho(j), rho0 )
-           FRrg_term=FRrg( temp(j), qr(j), rho(j) )
-           SHsr_term=CLcs_term+CLrs_term
-           if(temp(j)<=ti0)then
-              SHgr_term=CLcg_term+CLrg_term+CLig_term+CLsg_term-PG_term
-           else
-              SHgr_term=CLcg_term+CLrg_term
-           end if
-           CNcr_term=CNcr( 'B', temp(j), pres(j), qc(j), rho(j) )
-           CNis_term=CNis( temp(j), pres(j), qi(j), qs(j), qv(j),  &
-  &                  rho(j), rho0 )
-           CNsg_term=CNsg( temp(j), pres(j), qc(j), qs(j), rho(j), rho0 )
-           SPsi_term=SPxi( 's', temp(j), pres(j), qv(j), qc(j), qr(j), qi(j),  &
-  &                        qs(j), qg(j), rho(j), rho0 )
-           SPgi_term=SPxi( 'g', temp(j), pres(j), qv(j), qc(j), qr(j), qi(j),  &
-  &                        qs(j), qg(j), rho(j), rho0 )
-           ars=Category_rs( qr(j), qs(j), rho(j) )
-           forcept(j)=forcept(j)  &
-  &                   +Lv( temp(j) )*(VDvr_term)/(Cpd*exner_func_dry( pres(j) ))  &
-  &                   +(Ls( temp(j) )/(Cpd*exner_func_dry( pres(j) )))  &
-  &                   *(NUAvi_term+VDvi_term+VDvs_term+VDvg_term)  &
-  &                   +(Lf( temp(j) )/(Cpd*exner_func_dry( pres(j) )))  &
-  &                   *(NUFci_term+NUCci_term+CLcs_term+CLcg_term+CLri_term  &
-  &                    +CLrs_term+CLrg_term-MLic_term-MLsr_term-MLgr_term  &
-  &                    +FRrg_term-SHsr_term-SHgr_term)
-           forceqv(j)=forceqv(j)-NUAvi_term  &
-  &                             -VDvr_term-VDvi_term-VDvs_term-VDvg_term
-if(forceqv(j)/=0.0)then
-           write(*,*) "qv tend", NUAvi_term,  &
-  &                             -VDvr_term, -VDvi_term, -VDvs_term, -VDvg_term
-end if
-           forceqc(j)=forceqc(j)-NUFci_term-NUCci_term  &
-  &                             -CLcr_term-CLcs_term-CLcg_term-CLcr_term  &
-  &                             +MLic_term
-           forceqr(j)=forceqr(j)+VDvr_term+CLcr_term-CLri_term-CLrg_term  &
-  &                             +CNcr_term+MLsr_term+MLgr_term-FRrg_term  &
-  &                             +SHsr_term+SHgr_term
-           forceqi(j)=forceqi(j)+NUAvi_term+NUFci_term+NUCci_term  &
-  &                             +VDvi_term-CLir_term-CLis_term  &
-  &                             +CNis_term-MLic_term+SPsi_term+SPgi_term
-           forceqs(j)=forceqs(j)-SPsi_term+VDvs_term+CLcs_term+CLrs_term*ars  &
-  &                             +CLis_term-CLsr_term*(1.0-ars)-CLsg_term  &
-  &                             +CNis_term-CNsg_term-MLsr_term-SHsr_term
-!if(forceqs(j)/=0.0)then
-!write(*,*) -NUFci_term,-NUCci_term, -CLcr_term,-CLcs_term,-CLcg_term,-CLcr_term, +MLic_term
-!           write(*,*) -SPsi_term, +VDvs_term, +CLcs_term, +CLrs_term, ars,  &
-!  &                             +CLis_term, -CLsr_term*(1.0-ars), -CLsg_term,  &
-!  &                             +CNis_term, -CNsg_term, -MLsr_term, -SHsr_term
-!end if
-           forceqg(j)=forceqg(j)-SPgi_term+VDvg_term+PG_term  &
-  &                             +CLri_term+CLir_term  &
-  &                             +(CLrs_term+CLsr_term)*(1.0-ars)  &
-  &                             +CNsg_term-MLgr_term+FRrg_term-SHgr_term
-        end if
-     end do
-
   !-- adding the advecting terms
   !-- calculating vertical gradient
 
+     call grad_1d( z, pt, dptdz )
      call grad_1d( z, qv, dqvdz )
      call grad_1d( z, qc, dqcdz )
      call grad_1d( z, qr, dqrdz )
@@ -298,6 +220,7 @@ end if
      call grad_1d( z, fallqg, dfallqgdz )
 
      do j=2,nmax-1
+        forcept(j)=forcept(j)-w(j)*dptdz(j)
         forceqv(j)=forceqv(j)-w(j)*dqvdz(j)
         forceqc(j)=forceqc(j)-w(j)*dqcdz(j)
         forceqr(j)=forceqr(j)-w(j)*dqrdz(j)+dfallqrdz(j)/rho(j)
@@ -306,15 +229,15 @@ end if
         forceqg(j)=forceqg(j)-w(j)*dqgdz(j)+dfallqgdz(j)/rho(j)
      end do
 
-     if(i==1)then  ! first step is Euler method.
+     if(i==1)then
         do j=2,nmax-1
-           tmp_pt(j)=pt(j)+dt*forcept(j)
-           tmp_qv(j)=qv(j)+dt*forceqv(j)
-           tmp_qc(j)=qc(j)+dt*forceqc(j)
-           tmp_qr(j)=qr(j)+dt*forceqr(j)
-           tmp_qi(j)=qi(j)+dt*forceqi(j)
-           tmp_qs(j)=qs(j)+dt*forceqs(j)
-           tmp_qg(j)=qg(j)+dt*forceqg(j)
+           tmp_pt(j)=pt(j)+dtb*forcept(j)
+           tmp_qv(j)=qv(j)+dtb*forceqv(j)
+           tmp_qc(j)=qc(j)+dtb*forceqc(j)
+           tmp_qr(j)=qr(j)+dtb*forceqr(j)
+           tmp_qi(j)=qi(j)+dtb*forceqi(j)
+           tmp_qs(j)=qs(j)+dtb*forceqs(j)
+           tmp_qg(j)=qg(j)+dtb*forceqg(j)
         end do
 
         call forced_zero( tmp_qv )
@@ -324,6 +247,242 @@ end if
         call forced_zero( tmp_qs )
         call forced_zero( tmp_qg )
 
+     else  ! after step is leap flog method.
+
+        do j=2,nmax-1
+           tmp_pt(j)=old_pt(j)+dtb*forcept(j)
+           tmp_qv(j)=old_qv(j)+dtb*forceqv(j)
+           tmp_qc(j)=old_qc(j)+dtb*forceqc(j)
+           tmp_qr(j)=old_qr(j)+dtb*forceqr(j)
+           tmp_qi(j)=old_qi(j)+dtb*forceqi(j)
+           tmp_qs(j)=old_qs(j)+dtb*forceqs(j)
+           tmp_qg(j)=old_qg(j)+dtb*forceqg(j)
+        end do
+
+        call forced_zero( tmp_qv )
+        call forced_zero( tmp_qc )
+        call forced_zero( tmp_qr )
+        call forced_zero( tmp_qi )
+        call forced_zero( tmp_qs )
+        call forced_zero( tmp_qg )
+     end if
+
+  !-- calculating microphysics schemes
+
+     do j=2,nmax-1
+        coe1=1.0/(Cpd*exner_func_dry( pres(j) ))
+        VENTr_term=VENTx( 'r', temp(j), pres(j), qr(j), rho(j), rho0 ) 
+        VENTs_term=VENTx( 's', temp(j), pres(j), qs(j), rho(j), rho0 ) 
+        VENTg_term=VENTx( 'g', temp(j), pres(j), qg(j), rho(j), rho0 ) 
+        Gi_term=Gi( temp(j), pres(j), rho(j) ) 
+        Gw_term=Gw( temp(j), pres(j), rho(j) ) 
+        if(qv(j)>0.0.or.qc(j)>0.0.or.qr(j)>0.0)then
+     !-- First step (nucleation)
+           NUAvi_term=NUAvi( temp(j), pres(j), qv(j), qi(j), rho(j), dtb )
+           NUFci_term=NUFci( temp(j), qc(j), rho(j), dtb )
+           NUCci_term=NUCci( temp(j), pres(j), qc(j), rho(j) )
+           NUHci_term=NUHci( temp(j), qc(j), dtb )
+
+     !-- second step (collection)
+           CLcs_term=CLxy( 'cs', temp(j), pres(j), qc(j), qs(j), rho(j), rho0 )
+           CLcg_term=CLxy( 'cg', temp(j), pres(j), qc(j), qg(j), rho(j), rho0 )
+           CLri_term=CLxy( 'ri', temp(j), pres(j), qr(j), qi(j), rho(j), rho0 )
+           CLrs_term=CLxy( 'rs', temp(j), pres(j), qr(j), qs(j), rho(j), rho0 )
+           CLrg_term=CLxy( 'rg', temp(j), pres(j), qr(j), qg(j), rho(j), rho0 )
+           CLcr_term=CLxy( 'cr', temp(j), pres(j), qc(j), qr(j), rho(j), rho0 )
+           CLir_term=CLxy( 'ir', temp(j), pres(j), qi(j), qr(j), rho(j), rho0 )
+           CLis_term=CLxy( 'is', temp(j), pres(j), qi(j), qs(j), rho(j), rho0 )
+           CLig_term=CLxy( 'ig', temp(j), pres(j), qi(j), qg(j), rho(j), rho0 )
+           CLsr_term=CLxy( 'sr', temp(j), pres(j), qs(j), qr(j), rho(j), rho0 )
+           CLsg_term=CLxy( 'sg', temp(j), pres(j), qs(j), qg(j), rho(j), rho0 )
+
+     !-- CReSS original (CL adjust)
+           tmptend(1:3)=(/CLri_term, CLrs_term, CLrg_term/)
+           call redistCL( qr(j), tmptend(1:3) )
+           tmptend(1:2)=(/CLsr_term, CLsg_term/)
+           call redistCL( qs(j), tmptend(1:2) )
+           tmptend(1:3)=(/CLir_term, CLis_term, CLig_term/)
+           call redistCL( qi(j), tmptend(1:3) )
+
+     !-- third step (graupel production)
+           PGd_term=PGd( temp(j), pres(j), qv(j), qc(j), qr(j), qi(j),  &
+  &                      qs(j), qg(j), rho(j), rho0,  &
+  &                      opt_CLig=CLig_term, opt_CLsg=CLsg_term )
+           PGw_term=PGw( temp(j), pres(j), qv(j), qc(j), qr(j), qi(j),  &
+  &                      qs(j), qg(j), rho(j), rho0, dtb,  &
+  &                      opt_CLig=CLig_term, opt_CLsg=CLsg_term,  &
+  &                      opt_VENTg=VENTg_term )
+
+     !-- CReSS original (CL adjust)
+           if(PGw_term>0.0.and.PGw_term<PGd_term)then
+              tmptend(1:3)=(/CLir_term, CLis_term, CLig_term/)
+              call redistCL( qi(j), tmptend(1:3) )
+              tmptend(1:2)=(/CLsr_term, CLsg_term/)
+              call redistCL( qs(j), tmptend(1:2) )
+           else
+              PGw_term=-1.0
+           end if
+
+     !-- CReSS original (distpg) (in (4.32), calculating ars)
+           call distpg( temp(j), qr(j), qs(j), rho(j),  &
+  &                     CLrs_term, CLsr_term, CLrsg_term )
+
+     !-- fourth step (melting)
+           MLic_term=MLic( temp(j), qi(j), dtb )
+           MLsr_term=MLxr( 's', temp(j), pres(j), qv(j), qc(j), qr(j),  &
+  &                        qs(j), rho(j), rho0, opt_CLcx=CLcs_term,  &
+  &                        opt_CLrx=CLrs_term, opt_VENTx=VENTs_term )
+           MLgr_term=MLxr( 'g', temp(j), pres(j), qv(j), qc(j), qr(j),  &
+  &                        qg(j), rho(j), rho0, opt_CLcx=CLcg_term,  &
+  &                        opt_CLrx=CLrg_term, opt_VENTx=VENTg_term )
+
+     !-- fifth step (deposition)
+           VDvr_term=VDvx( 'r', temp(j), pres(j), qv(j), qc(j), qr(j),  &
+  &                        qr(j), rho(j), rho0, dtb, opt_CLcx=CLcr_term,  &
+  &                        opt_VENTx=VENTr_term,  &
+  &                        opt_Gw=Gw_term, opt_Gi=Gi_term )
+           VDvi_term=VDvi( temp(j), pres(j), qv(j), qi(j), rho(j) )
+           VDvs_term=VDvx( 's', temp(j), pres(j), qv(j), qc(j), qr(j),  &
+  &                        qs(j), rho(j), rho0, dtb, opt_CLcx=CLcs_term,  &
+  &                        opt_MLxr=MLsr_term, opt_VENTx=VENTs_term,  &
+  &                        opt_Gw=Gw_term, opt_Gi=Gi_term )
+           VDvg_term=VDvx( 'g', temp(j), pres(j), qv(j), qc(j), qr(j),  &
+  &                        qg(j), rho(j), rho0, dtb, opt_CLcx=CLcg_term,  &
+  &                        opt_MLxr=MLgr_term, opt_VENTx=VENTg_term,  &
+  &                        opt_Gw=Gw_term, opt_Gi=Gi_term )
+
+     !-- CReSS original (VD adjust)
+           tmptend(1:3)=(/VDvi_term, VDvs_term, VDvg_term/)
+           call redistVD( (qv(j)-qvsi(temp(j),pres(j))), tmptend(1:3) )
+
+     !-- sixth step (conversion)
+           CNcr_term=CNcr( 'B', temp(j), pres(j), qc(j), rho(j) )
+           CNis_term=CNis( temp(j), pres(j), qi(j), qs(j), qv(j),  &
+  &                        rho(j), rho0, opt_VDvi=VDvi_term )
+           CNsg_term=CNsg( temp(j), pres(j), qc(j), qs(j), rho(j), rho0, dtb )
+
+     !-- seventh step (2nd nucleation)
+           SPsi_term=SPxi( 's', temp(j), pres(j), qv(j), qc(j), qr(j), qi(j),  &
+  &                        qs(j), qg(j), rho(j), rho0, dtb,  &
+  &                        opt_CLcx=CLcs_term )
+           SPgi_term=SPxi( 'g', temp(j), pres(j), qv(j), qc(j), qr(j), qi(j),  &
+  &                        qs(j), qg(j), rho(j), rho0, dtb,  &
+  &                        opt_CLcx=CLcg_term, opt_PGw=PGw_term,  &
+  &                        opt_CLig=CLig_term, opt_CLsg=CLsg_term,  &
+  &                        opt_VENTg=VENTg_term )
+
+     !-- eighth step (rain freezing)
+           FRrg_term=FRrg( temp(j), qr(j), rho(j), dtb )
+
+     !-- nineth step (shedding)
+           SHsr_term=SHsr( temp(j), pres(j), qc(j), qr(j), qs(j), rho(j),  &
+  &                        rho0, opt_CLcs=CLcs_term, opt_CLrs=CLrs_term )
+           SHgr_term=SHgr( temp(j), pres(j), qv(j), qc(j), qr(j), qi(j),  &
+  &                        qs(j), qg(j), rho(j), rho0, opt_CLcg=CLcg_term,  &
+  &                        opt_CLrg=CLrg_term,  &
+  &                        opt_CLig=CLig_term, opt_CLsg=CLsg_term,  &
+  &                        opt_PGw=PGw_term )
+
+!           ars=Category_rs( qr(j), qs(j), rho(j) )
+
+     !-- forcing order ( pt -> qr -> qc -> pt(ad) -> qi -> pt(ad) -> qv )
+           forcept(j)=+(Lv( temp(j) )*(VDvr_term)  &
+  &                   +Ls( temp(j) )*(NUAvi_term+VDvi_term+VDvs_term+VDvg_term)  &
+  &                   +Lf( temp(j) )*(NUFci_term+NUCci_term  &
+  &                                  +CLcs_term+CLcg_term+CLri_term  &
+  &                                  +CLrs_term+CLrg_term-MLic_term  &
+  &                                  -MLsr_term-MLgr_term+FRrg_term  &
+  &                                  -SHsr_term-SHgr_term+CLrsg_term))*coe1
+
+           forceqv(j)=-NUAvi_term  &
+  &                   -VDvr_term-VDvi_term-VDvs_term-VDvg_term
+           forceqc(j)=-NUFci_term-NUCci_term  &
+  &                   -CLcr_term-CLcs_term-CLcg_term-CNcr_term  &
+  &                   +MLic_term
+           forceqr(j)=+VDvr_term+CLcr_term-CLri_term-CLrs_term-CLrg_term  &
+  &                   +CNcr_term+MLsr_term+MLgr_term-FRrg_term  &
+  &                   +SHsr_term+SHgr_term-CLrsg_term
+           forceqi(j)=+NUAvi_term+NUFci_term+NUCci_term  &
+  &                   +VDvi_term-CLir_term-CLis_term-CLig_term  &
+  &                   -CNis_term-MLic_term+SPsi_term+SPgi_term
+           forceqs(j)=-SPsi_term+VDvs_term+CLcs_term+CLrs_term  &
+  &                   +CLis_term-CLsr_term-CLsg_term  &
+  &                   +CNis_term-CNsg_term-MLsr_term-SHsr_term
+           forceqg(j)=-SPgi_term+VDvg_term  &
+  &                   +CLri_term+CLir_term+CLrsg_term+CLsr_term  &
+  &                   +CLsg_term+CNsg_term-MLgr_term+FRrg_term-SHgr_term  &
+  &                   +CLcg_term+CLrg_term+CLig_term
+
+           !-- pt time integration
+           tmp_pt(j)=tmp_pt(j)+dtb*forcept(j)
+           !-- qr time integration
+           if(tmp_qr(j)<-dtb*forceqr(j))then
+              forceqr(j)=forceqr(j)+tmp_qr(j)/dtb
+              tmp_qr(j)=0.0
+              forceqc(j)=forceqc(j)+forceqr(j)
+           else
+              tmp_qr(j)=tmp_qr(j)+dtb*forceqr(j)
+           end if
+
+           !-- qc time integration
+           if(tmp_qc(j)<-dtb*forceqc(j))then
+              forceqc(j)=forceqc(j)+tmp_qc(j)/dtb
+              tmp_pt(j)=tmp_pt(j)-Lv( temp(j) )*forceqc(j)*coe1
+              forceqv(j)=forceqv(j)+forceqc(j)
+              tmp_qc(j)=0.0
+           else
+              tmp_qc(j)=tmp_qc(j)+dtb*forceqc(j)
+           end if
+
+           !-- qi time integration
+           if(tmp_qi(j)<-dtb*forceqi(j))then
+              forceqi(j)=forceqi(j)+tmp_qi(j)/dtb
+              tmp_pt(j)=tmp_pt(j)-Ls( temp(j) )*forceqi(j)*coe1
+              forceqv(j)=forceqv(j)+forceqi(j)
+              tmp_qi(j)=0.0
+           else
+              tmp_qi(j)=tmp_qi(j)+dtb*forceqi(j)
+           end if
+
+           !-- qv time integration
+           if(tmp_qv(j)<-dtb*forceqv(j))then
+              tmp_qv(j)=0.0
+           else
+              tmp_qv(j)=tmp_qv(j)+dtb*forceqv(j)
+           end if
+
+           !-- qs time integration
+           tmp_qs(j)=tmp_qs(j)+dtb*forceqs(j)
+           !-- qg time integration
+           tmp_qg(j)=tmp_qg(j)+dtb*forceqg(j)
+
+!if(forceqs(j)/=0.0)then
+!write(*,*) -NUFci_term,-NUCci_term, -CLcr_term,-CLcs_term,-CLcg_term,-CLcr_term, +MLic_term
+!           write(*,*) -SPsi_term, +VDvs_term, +CLcs_term, +CLrs_term, ars,  &
+!  &                             +CLis_term, -CLsr_term*(1.0-ars), -CLsg_term,  &
+!  &                             +CNis_term, -CNsg_term, -MLsr_term, -SHsr_term
+!end if
+        end if
+     end do
+
+     if(i==1)then  ! first step is Euler method.
+!        do j=2,nmax-1
+!           tmp_pt(j)=pt(j)+dt*forcept(j)
+!           tmp_qv(j)=qv(j)+dt*forceqv(j)
+!           tmp_qc(j)=qc(j)+dt*forceqc(j)
+!           tmp_qr(j)=qr(j)+dt*forceqr(j)
+!           tmp_qi(j)=qi(j)+dt*forceqi(j)
+!           tmp_qs(j)=qs(j)+dt*forceqs(j)
+!           tmp_qg(j)=qg(j)+dt*forceqg(j)
+!        end do
+
+        call forced_zero( tmp_qv )
+        call forced_zero( tmp_qc )
+        call forced_zero( tmp_qr )
+        call forced_zero( tmp_qi )
+        call forced_zero( tmp_qs )
+        call forced_zero( tmp_qg )
+
         !-- Moist Saturated Adjustment
 
         do j=2,nmax-1
@@ -398,15 +557,15 @@ end if
 
      else  ! after step is leap flog method.
 
-        do j=2,nmax-1
-           tmp_pt(j)=old_pt(j)+2.0*dt*forcept(j)
-           tmp_qv(j)=old_qv(j)+2.0*dt*forceqv(j)
-           tmp_qc(j)=old_qc(j)+2.0*dt*forceqc(j)
-           tmp_qr(j)=old_qr(j)+2.0*dt*forceqr(j)
-           tmp_qi(j)=old_qi(j)+2.0*dt*forceqi(j)
-           tmp_qs(j)=old_qs(j)+2.0*dt*forceqs(j)
-           tmp_qg(j)=old_qg(j)+2.0*dt*forceqg(j)
-        end do
+!        do j=2,nmax-1
+!           tmp_pt(j)=old_pt(j)+2.0*dt*forcept(j)
+!           tmp_qv(j)=old_qv(j)+2.0*dt*forceqv(j)
+!           tmp_qc(j)=old_qc(j)+2.0*dt*forceqc(j)
+!           tmp_qr(j)=old_qr(j)+2.0*dt*forceqr(j)
+!           tmp_qi(j)=old_qi(j)+2.0*dt*forceqi(j)
+!           tmp_qs(j)=old_qs(j)+2.0*dt*forceqs(j)
+!           tmp_qg(j)=old_qg(j)+2.0*dt*forceqg(j)
+!        end do
 
         call forced_zero( tmp_qv )
         call forced_zero( tmp_qc )
@@ -418,11 +577,16 @@ end if
         !-- Moist Saturated Adjustment
 
         do j=2,nmax-1
-           if(tmp_qv(j)>0.0.or.tmp_qi(j)>0.0)then
-              call Moist_Sature_Adjust_Ice( pres(j), tmp_pt(j), tmp_qv(j), tmp_qi(j) )
+           if(tmp_qv(j)>=0.0.and.tmp_qi(j)>=0.0)then
+              if((tmp_qv(j)/=0.0.or.tmp_qi(j)/=0.0)  &
+  &              .and.(temp(j)<tscp))then
+                 call Moist_Sature_Adjust_Ice( pres(j), tmp_pt(j), tmp_qv(j), tmp_qi(j) )
+              end if
            end if
-           if(tmp_qv(j)>0.0.or.tmp_qc(j)>0.0)then
-              call Moist_Sature_Adjust( pres(j), tmp_pt(j), tmp_qv(j), tmp_qc(j) )
+           if(tmp_qv(j)>=0.0.and.tmp_qc(j)>=0.0)then
+              if((tmp_qv(j)/=0.0.or.tmp_qc(j)/=0.0))then
+                 call Moist_Sature_Adjust( pres(j), tmp_pt(j), tmp_qv(j), tmp_qc(j) )
+              end if
            end if
         end do
 
