@@ -109,9 +109,11 @@ write(*,*) "nl check", nl
   &                     omega_fix=omegar, omegan_fix=omeganr )
 !     call c2r_ffttp_1d( nl, co, ren(1:nl,j), prim='o', prim_fact=prim,  &
 !  &                     omega_fix=omegai, omegan_fix=omegani )
-     do i=1,nl/2
+     do i=2,nl/2
         sp(i,j)=abs(co(i,j))
      end do
+     sp(1,j)=abs(real(co(1,j)))
+     sp(nl/2+1,j)=abs(aimag(co(1,j)))
   end do
 
   write(*,*) "### Finished calculating."
@@ -143,7 +145,7 @@ write(*,*) "check a format", trim(fom)
      write(11,trim(fomc)) trim(ctitle)
 
 !-- j=1 ʿѾɽΤ, ƽϤʤ.
-     do j=2,nl/2
+     do j=1,nl/2+1
         write(11,trim(fom)) real(j-1), real(j-1)/period, (sp(j,i),i=1,na)
      end do
 
