@@ -350,7 +350,7 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
      do k=1,nz
         do i=1,nx
            call grad_1d( y, tmpv(i,:,k), dvdy(i,:,k), undeff )
@@ -360,7 +360,7 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
      do k=1,nz
         do j=1,ny
            call grad_1d( x, tmpu(:,j,k), dudx(:,j,k), undeff )
@@ -370,7 +370,7 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do j=1,ny
         do i=1,nx
            call grad_1d( z, tmpw(i,j,:), dwdz(i,j,:), undeff )
@@ -405,7 +405,7 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
   else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j,k)
+!$omp do schedule(runtime) private(i,j,k)
      do k=1,nz
         do j=1,ny
            do i=1,nx
@@ -419,7 +419,7 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
      do k=1,nz
         do i=1,nx
            call grad_1d( y, tmpv(i,:,k), dvdy(i,:,k) )
@@ -429,7 +429,7 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
      do k=1,nz
         do j=1,ny
            call grad_1d( x, tmpu(:,j,k), dudx(:,j,k) )
@@ -439,7 +439,7 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do j=1,ny
         do i=1,nx
            call grad_1d( z, tmpw(i,j,:), dwdz(i,j,:) )
@@ -531,7 +531,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
   if(present(undeff))then
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i)
+!$omp do schedule(runtime) private(i)
 ! x ˿ľ̾Ǥ x α٥٥ȥ 3 Ƿ׻.
      do i=1,nx
         call curl( y, z, v(i,:,:), w(i,:,:), zeta(i,:,:), undeff,  &
@@ -543,7 +543,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
   else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i)
+!$omp do schedule(runtime) private(i)
 ! x ˿ľ̾Ǥ x α٥٥ȥ 3 Ƿ׻.
      do i=1,nx
         call curl( y, z, v(i,:,:), w(i,:,:), zeta(i,:,:),  &
@@ -559,7 +559,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
   if(present(undeff))then
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j)
+!$omp do schedule(runtime) private(j)
      do j=1,ny
         call curl( x, z, u(:,j,:), w(:,j,:), eta(:,j,:), undeff, ord=.false., &
   &                hx=scalex(:,j,:), hy=scalez(:,j,:) )
@@ -570,7 +570,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
   else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j)
+!$omp do schedule(runtime) private(j)
      do j=1,ny
         call curl( x, z, u(:,j,:), w(:,j,:), eta(:,j,:), ord=.false.,  &
   &                hx=scalex(:,j,:), hy=scalez(:,j,:) )
@@ -585,7 +585,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
   if(present(undeff))then
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(k)
+!$omp do schedule(runtime) private(k)
      do k=1,nz
         call curl( x, y, u(:,:,k), v(:,:,k), xi(:,:,k), undeff,  &
   &                hx=scalex(:,:,k), hy=scaley(:,:,k) )
@@ -596,7 +596,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
   else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(k)
+!$omp do schedule(runtime) private(k)
      do k=1,nz
         call curl( x, y, u(:,:,k), v(:,:,k), xi(:,:,k),  &
   &                hx=scalex(:,:,k), hy=scaley(:,:,k) )
@@ -856,7 +856,7 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
 
   if(present(undeff))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do i=1,nz
         do j=1,ny
            call grad_1d(x, u(:,j,i), valx(:,j,i),undeff)
@@ -866,7 +866,7 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do i=1,nz
         do j=1,nx
            call grad_1d(y, u(j,:,i), valy(j,:,i),undeff)
@@ -876,7 +876,7 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do i=1,ny
         do j=1,nx
            call grad_1d(z, u(j,i,:), valz(j,i,:),undeff)
@@ -916,7 +916,7 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
   else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do i=1,nz
         do j=1,ny
            call grad_1d(x, u(:,j,i), valx(:,j,i))
@@ -926,7 +926,7 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do i=1,nz
         do j=1,nx
            call grad_1d(y, u(j,:,i), valy(j,:,i))
@@ -936,7 +936,7 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do i=1,ny
         do j=1,nx
            call grad_1d(z, u(j,i,:), valz(j,i,:))
@@ -1212,48 +1212,48 @@ subroutine grad4_2d( x, y, u, valx, valy, undeff, hx, hy )
 
 end subroutine grad4_2d
 
+!-----------------------------------------
+!-----------------------------------------
 
-
-subroutine zast_2_vel_2d( x, y, zast, u, v, w, uh, vh, wh, undef )
-! terrain following ɸϤɾƤ®ʿȱľѴ.
-! , 3 ǥȷϤѴΤǤϤʤ, terrain following ɸϤ
+subroutine zast_2_w_2d( x, y, zeta, zf, zt, u, v, w, wh, undef )
+! terrain following ɸϤɾƤ®ʬǥȺɸϤǤ
+! ®ʬѴ.
+! , terrain following Ϥοʿ®ϥǥȺɸϤǤ
+! 礭Ѳʤ, Υ롼Ǥϱľ®ΤѴԤ.
+! ޤ, 3 ǥȷϤγʻѴΤǤϤʤ, terrain following Ϥ
 ! ®ʬǥȷѴ.
 ! , ʿˤϥǥȷϤˤΤбƤ.
 ! ⤷, ɸѴ, vert_coord_conv ⥸塼ѤΤ.
   implicit none
   real, dimension(:), intent(in) :: x  ! κɸ [m]
   real, dimension(:), intent(in) :: y  ! κɸ [m]
-  real, dimension(size(x),size(y)), intent(in) :: zast  ! terrain Ϥ,  1  [m]
-  real, dimension(size(x),size(y)), intent(in) :: u  ! zast ˤ [m/s]
-  real, dimension(size(x),size(y)), intent(in) :: v  ! zast ˤ [m/s]
-  real, dimension(size(x),size(y)), intent(in) :: w  ! zast ˤľ [m/s]
-  real, dimension(size(x),size(y)), intent(inout) :: uh  ! ǥȷϤˤ [m/s]
-  real, dimension(size(x),size(y)), intent(inout) :: vh  ! ǥȷϤˤ [m/s]
+  real, dimension(size(x),size(y)), intent(in) :: zeta  ! terrain ϱľɸ [m]
+  real, dimension(size(x),size(y)), intent(in) :: zf  ! ɽ̹ٺɸ [m]
+  real, dimension(size(x),size(y)), intent(in) :: zt  ! ɸǾü [m]
+  real, dimension(size(x),size(y)), intent(in) :: u  ! zeta ˤ [m/s]
+  real, dimension(size(x),size(y)), intent(in) :: v  ! zeta ˤ [m/s]
+  real, dimension(size(x),size(y)), intent(in) :: w  ! zeta ˤľ [m/s]
   real, dimension(size(x),size(y)), intent(inout) :: wh  ! ǥȷϤˤľ [m/s]
   real, intent(in), optional :: undef  ! »
   integer :: i, j, nx, ny
   real, dimension(size(x),size(y)) :: dx, dy
-  real :: cosx, sinx, cosy, siny
+  real :: j31, j32, jd, coe
 
   nx=size(x)
   ny=size(y)
 
-  call grad_2d( x, y, zast, dx, dy )
+  call grad_2d( x, y, zf, dx, dy )
 
   if(present(undef))then
      do j=1,ny
         do i=1,nx
            if(u(i,j)/=undef.and.v(i,j)/=undef.and.w(i,j)/=undef)then
-              cosx=1.0/sqrt(1.0+dx(i,j)*dx(i,j))
-              cosy=1.0/sqrt(1.0+dy(i,j)*dy(i,j))
-              sinx=dx(i,j)*cosx
-              siny=dy(i,j)*cosy
-              uh(i,j)=u(i,j)*cosx
-              vh(i,j)=v(i,j)*cosy
-              wh(i,j)=w(i,j)+u(i,j)*sinx+v(i,j)*siny
+              jd=1.0-zf(i,j)/zt(i,j)
+              coe=zeta(i,j)/zt(i,j)-1.0
+              j31=coe*dx(i,j)
+              j32=coe*dy(i,j)
+              wh(i,j)=(u(i,j)*j31+v(i,j)*j32+w(i,j))/jd
            else
-              uh(i,j)=undef
-              vh(i,j)=undef
               wh(i,j)=undef
            end if
         end do
@@ -1261,52 +1261,54 @@ subroutine zast_2_vel_2d( x, y, zast, u, v, w, uh, vh, wh, undef )
   else
      do j=1,ny
         do i=1,nx
-           cosx=1.0/sqrt(1.0+dx(i,j)*dx(i,j))
-           cosy=1.0/sqrt(1.0+dy(i,j)*dy(i,j))
-           sinx=dx(i,j)*cosx
-           siny=dy(i,j)*cosy
-           uh(i,j)=u(i,j)*cosx
-           vh(i,j)=v(i,j)*cosy
-           wh(i,j)=w(i,j)+u(i,j)*sinx+v(i,j)*siny
+           jd=1.0-zf(i,j)/zt(i,j)
+           coe=zeta(i,j)/zt(i,j)-1.0
+           j31=coe*dx(i,j)
+           j32=coe*dy(i,j)
+           wh(i,j)=(u(i,j)*j31+v(i,j)*j32+w(i,j))/jd
         end do
      end do
   end if
 
-end subroutine zast_2_vel_2d
+end subroutine zast_2_w_2d
 
 !-----------------------------------------
 !-----------------------------------------
 
-subroutine zast_2_vel_3d( x, y, zast, u, v, w, uh, vh, wh, undef )
-! terrain following ɸϤɾƤ®ʿȱľѴ.
-! , 3 ǥȷϤѴΤǤϤʤ, terrain following ɸϤ
+subroutine zast_2_w_3d( x, y, zeta, zf, zt, u, v, w, wh, undef )
+! terrain following ɸϤɾƤ®ʬǥȺɸϤǤ
+! ®ʬѴ.
+! , terrain following Ϥοʿ®ϥǥȺɸϤǤ
+! 礭Ѳʤ, Υ롼Ǥϱľ®ΤѴԤ.
+! ޤ, 3 ǥȷϤγʻѴΤǤϤʤ, terrain following Ϥ
 ! ®ʬǥȷѴ.
 ! , ʿˤϥǥȷϤˤΤбƤ.
 ! ⤷, ɸѴ, vert_coord_conv ⥸塼ѤΤ.
   implicit none
-  real, dimension(:,:,:), intent(in) :: zast  ! terrain Ϥιٺɸ [m]
-  real, dimension(size(zast,1)), intent(in) :: x  ! κɸ [m]
-  real, dimension(size(zast,2)), intent(in) :: y  ! κɸ [m]
-  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: u  ! zast ˤ [m/s]
-  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: v  ! zast ˤ [m/s]
-  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: w  ! zast ˤľ [m/s]
-  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: uh  ! ǥȷϤˤ [m/s]
-  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: vh  ! ǥȷϤˤ [m/s]
-  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: wh  ! ǥȷϤˤľ [m/s]
+  real, dimension(:,:,:), intent(in) :: zeta  ! terrain Ϥαľɸ [m]
+  real, dimension(size(zeta,1)), intent(in) :: x  ! κɸ [m]
+  real, dimension(size(zeta,2)), intent(in) :: y  ! κɸ [m]
+  real, dimension(size(zeta,1),size(zeta,2)), intent(in) :: zf  ! ɽ̹ٺɸ [m]
+  real, dimension(size(zeta,1),size(zeta,2)), intent(in) :: zt  ! ɸǾü [m]
+  real, dimension(size(zeta,1),size(zeta,2),size(zeta,3)), intent(in) :: u  ! zeta ˤ [m/s]
+  real, dimension(size(zeta,1),size(zeta,2),size(zeta,3)), intent(in) :: v  ! zeta ˤ [m/s]
+  real, dimension(size(zeta,1),size(zeta,2),size(zeta,3)), intent(in) :: w  ! zeta ˤľ [m/s]
+  real, dimension(size(zeta,1),size(zeta,2),size(zeta,3)), intent(inout) :: wh  ! ǥȷϤˤľ [m/s]
   real, intent(in), optional :: undef  ! »
   integer :: k, nx, ny, nz
 
-  nx=size(zast,1)
-  ny=size(zast,2)
-  nz=size(zast,3)
+  nx=size(zeta,1)
+  ny=size(zeta,2)
+  nz=size(zeta,3)
 
   if(present(undef))then
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(k)
+!$omp do schedule(runtime) private(k)
      do k=1,nz
-        call zast_2_vel_2d( x, y, zast(:,:,k), u(:,:,k), v(:,:,k), w(:,:,k),  &
-  &                         uh(:,:,k), vh(:,:,k), wh(:,:,k), undef )
+        call zast_2_w_2d( x, y, zeta(:,:,k), zf, zt,  &
+  &                       u(:,:,k), v(:,:,k), w(:,:,k),  &
+  &                       wh(:,:,k), undef )
      end do
 !$omp end do
 !$omp end parallel
@@ -1314,17 +1316,18 @@ subroutine zast_2_vel_3d( x, y, zast, u, v, w, uh, vh, wh, undef )
   else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(k)
+!$omp do schedule(runtime) private(k)
      do k=1,nz
-        call zast_2_vel_2d( x, y, zast(:,:,k), u(:,:,k), v(:,:,k), w(:,:,k),  &
-  &                         uh(:,:,k), vh(:,:,k), wh(:,:,k) )
+        call zast_2_w_2d( x, y, zeta(:,:,k), zf, zt,  &
+  &                       u(:,:,k), v(:,:,k), w(:,:,k),  &
+  &                       wh(:,:,k) )
      end do
 !$omp end do
 !$omp end parallel
 
   end if
 
-end subroutine zast_2_vel_3d
+end subroutine zast_2_w_3d
 
 !-----------------------------------------
 !-----------------------------------------
@@ -1383,60 +1386,201 @@ end subroutine laplacian_1d
 
 !-----------------------------------------
 !-----------------------------------------
-!
-!subroutine laplacian_2d( x, y, u, val, undef )
-!! 1 Υ顼ѿΥץ饷׻
-!! $\frac{\partial ^2p}{\partial x^2} $ 
-!! 2 ʬǽ񤭴,  $(i)$ Ǥθۤ
-!! $\frac{p_{i+1}+p_{i-1}-2p_i}{dx^2} $
-!! ȤǤ. Ѥ 1 ץ饷׻.
-!! ǡ­ʤΰǤ, undef .
-!! option  undef Ƥʤ, 0.0 .
-!  implicit none
-!  real, intent(in) :: x(:)  ! x κɸѿ [m]
-!  real, intent(in) :: u(size(x))  ! ζб 1 顼
-!  real, intent(inout) :: val(size(x))  ! 顼ͤ x θ
-!  real, intent(in), optional :: undef
-!  integer :: i  ! 졼ź
-!  integer :: nx  ! ǿ
-!  real :: scalex(size(x))
-!  real :: tmpu(size(x))
-!
-!  nx=size(x)
-!
-!  if(present(undef))then
-!     do i=2,nx-1
-!        if(u(i+1)==undef.or.u(i-1)==undef.or.u(i)==undef)then
-!           val(i)=undef
-!        else
-!           val(i)=4.0*(u(i+1)+u(i-1)-2.0*u(i))/((x(i+1)-x(i-1))**2)
-!        end if
-!     end do
-!!-- ǡΤʤξüν ---
-!!     if(u(1)==undef.or.u(2)==undef)then
-!!        val(1)=undef
-!!     else
-!!        val(1)=(u(2)-u(1))/dx
-!!     end if
-!!     if(u(nx)==undef.or.u(nx-1)==undef)then
-!!        val(nx)=undef
-!!     else
-!!        val(nx)=(u(nx)-u(nx-1))/dx
-!!     end if
-!!     val(1)=undef
-!!     val(nx)=undef
-!!  else
-!!     do i=2,nx-1
-!!        val(i)=4.0*(u(i+1)+u(i-1)-2.0*u(i))/((x(i+1)-x(i-1))**2)
-!!     end do
-!!-- ǡΤʤξüν ---
-!!     val(1)=(u(2)-u(1))/dx
-!!     val(nx)=(u(nx)-u(nx-1))/dx
-!     val(1)=0.0
-!     val(nx)=0.0
-!  end if
-!
-!end subroutine laplacian_2d
+
+subroutine laplacian_2d( x, y, u, val, undef )
+! 2 Υ顼ѿΥץ饷׻
+! $\frac{\partial ^2p}{\partial x^2} $ 
+! 2 ʬǽ񤭴,  $(i)$ Ǥθۤ
+! $\frac{p_{i+1}+p_{i-1}-2p_i}{dx^2} $
+! ȤǤ. Ѥ 1 ץ饷׻.
+! ǡ­ʤΰǤ, undef .
+! option  undef Ƥʤ, 0.0 .
+! ľɸϤؤбϤޤ.
+  implicit none
+  real, intent(in) :: x(:)  ! x κɸѿ [m]
+  real, intent(in) :: y(:)  ! y κɸѿ [m]
+  real, intent(in) :: u(size(x),size(y))  ! ζб 2 顼
+  real, intent(inout) :: val(size(x),size(y))  ! 顼ͤ 2 Υץ饷
+  real, intent(in), optional :: undef
+  integer :: i, j  ! 졼ź
+  integer :: nx, ny  ! ǿ
+!  real :: scalex(size(x),size(y))
+  real :: tmpu(size(x),size(y)), tmpv(size(x),size(y))
+
+  nx=size(x)
+  ny=size(y)
+
+  if(present(undef))then
+
+     val=undef
+
+     do j=1,ny
+        call laplacian_1d( x, u(1:nx,j), tmpu(1:nx,j), undef )
+     end do
+     do i=1,nx
+        call laplacian_1d( y, u(i,1:ny), tmpv(i,1:ny), undef )
+     end do
+
+     do j=1,ny
+        do i=1,nx
+           if(tmpu(i,j)/=undef.and.tmpv(i,j)/=undef)then
+              val(i,j)=tmpu(i,j)+tmpv(i,j)
+           end if
+        end do
+     end do
+
+  else
+
+     do j=1,ny
+        call laplacian_1d( x, u(1:nx,j), tmpu(1:nx,j) )
+     end do
+     do i=1,nx
+        call laplacian_1d( y, u(i,1:ny), tmpv(i,1:ny) )
+     end do
+
+     do j=1,ny
+        do i=1,nx
+           val(i,j)=tmpu(i,j)+tmpv(i,j)
+        end do
+     end do
+
+  end if
+
+end subroutine laplacian_2d
+
+!-----------------------------------------
+!-----------------------------------------
+
+subroutine laplacian_3d( x, y, z, u, val, undef )
+! 3 Υ顼ѿΥץ饷׻
+! $\frac{\partial ^2p}{\partial x^2} $ 
+! 2 ʬǽ񤭴,  $(i)$ Ǥθۤ
+! $\frac{p_{i+1}+p_{i-1}-2p_i}{dx^2} $
+! ȤǤ. Ѥ 1 ץ饷׻.
+! ǡ­ʤΰǤ, undef .
+! option  undef Ƥʤ, 0.0 .
+! ľɸϤؤбϤޤ.
+  implicit none
+  real, intent(in) :: x(:)  ! x κɸѿ [m]
+  real, intent(in) :: y(:)  ! y κɸѿ [m]
+  real, intent(in) :: z(:)  ! z κɸѿ [m]
+  real, intent(in) :: u(size(x),size(y),size(z))  ! ζб 3 顼
+  real, intent(inout) :: val(size(x),size(y),size(z))  ! 顼ͤ 3 Υץ饷
+  real, intent(in), optional :: undef
+  integer :: i, j, k  ! 졼ź
+  integer :: nx, ny, nz  ! ǿ
+!  real :: scalex(size(x),size(y))
+  real :: tmpu(size(x),size(y),size(z)), tmpv(size(x),size(y),size(z))
+
+  nx=size(x)
+  ny=size(y)
+  nz=size(z)
+
+  if(present(undef))then
+
+     val=undef
+
+!$omp parallel default(shared)
+!$omp do schedule(dynamic) private(k)
+
+     do k=1,nz
+        call laplacian_2d( x, y, u(1:nx,1:ny,k), tmpu(1:nx,1:ny,k), undef )
+     end do
+
+!$omp end do
+!$omp end parallel
+
+     do j=1,ny
+        do i=1,nx
+           call laplacian_1d( z, u(i,j,1:nz), tmpv(i,j,1:nz), undef )
+        end do
+     end do
+
+!$omp parallel default(shared)
+!$omp do schedule(dynamic) private(i,j,k)
+
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              if(tmpu(i,j,k)/=undef.and.tmpv(i,j,k)/=undef)then
+                 val(i,j,k)=tmpu(i,j,k)+tmpv(i,j,k)
+              end if
+           end do
+        end do
+     end do
+
+!$omp end do
+!$omp end parallel
+
+  else
+
+!$omp parallel default(shared)
+!$omp do schedule(dynamic) private(k)
+
+     do k=1,nz
+        call laplacian_2d( x, y, u(1:nx,1:ny,k), tmpu(1:nx,1:ny,k) )
+     end do
+
+!$omp end do
+!$omp end parallel
+
+     do j=1,ny
+        do i=1,nx
+           call laplacian_1d( z, u(i,j,1:nz), tmpv(i,j,1:nz) )
+        end do
+     end do
+
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(i,j,k)
+
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              val(i,j,k)=tmpu(i,j,k)+tmpv(i,j,k)
+           end do
+        end do
+     end do
+
+!$omp end do
+!$omp end parallel
+
+  end if
+
+end subroutine laplacian_3d
+
+!-----------------------------------------
+!-----------------------------------------
+
+subroutine z_2_zeta( z, zf, zt, zeta )
+! ɸ z  terrain following ɸѴ.
+  implicit none
+  real, intent(in) :: z(:,:,:)    ! ǥȴɸϱľɸ [m]
+  real, intent(in) :: zf(size(z,1),size(z,2))   ! ɽ̹ [m]
+  real, intent(in) :: zt(size(z,1),size(z,2))   ! ǹ [m]
+  real, intent(inout) :: zeta(size(z,1),size(z,2),size(z,3))  ! terrain following ɸ [m]
+
+  integer :: i, j, k
+  integer :: nx, ny, nz
+
+  nx=size(z,1)
+  ny=size(z,2)
+  nz=size(z,3)
+
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(i,j,k)
+
+  do k=1,nz
+     do j=1,ny
+        do i=1,nx
+           zeta(i,j,k)=zt(i,j)*(z(i,j,k)-zf(i,j))/(zt(i,j)-zf(i,j))
+        end do
+     end do
+  end do
+
+!$omp end do
+!$omp end parallel
+
+end subroutine z_2_zeta
 
 !-----------------------------------------
 !-----------------------------------------
@@ -1897,7 +2041,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do j=1,ny
               call grad_1d( x, tmpv(:,j,k), ddx(:,j,k), undef=undef )
@@ -1907,7 +2051,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
         do k=1,nz
            do i=1,nx
               call grad_1d( y, tmpu(i,:,k), ddy(i,:,k), undef=undef )
@@ -1929,7 +2073,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do j=1,ny
               call grad_1d( x, tmpv(:,j,k), ddx(:,j,k) )
@@ -1939,7 +2083,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do i=1,nx
               call grad_1d( y, tmpu(i,:,k), ddy(i,:,k) )
@@ -1989,7 +2133,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
         do k=1,nz
            do i=1,nx
               call grad_1d( y, tmpw(i,:,k), ddy(i,:,k), undef=undef )
@@ -1999,7 +2143,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
         do j=1,ny
            do i=1,nx
               call grad_1d( z, tmpv(i,j,:), ddz(i,j,:), undef=undef )
@@ -2021,7 +2165,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
         do k=1,nz
            do i=1,nx
               call grad_1d( y, tmpw(i,:,k), ddy(i,:,k) )
@@ -2031,7 +2175,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
         do j=1,ny
            do i=1,nx
               call grad_1d( z, tmpv(i,j,:), ddz(i,j,:) )
@@ -2081,7 +2225,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
         do j=1,ny
            do i=1,nx
               call grad_1d( z, tmpu(i,j,:), ddz(i,j,:), undef=undef )
@@ -2091,7 +2235,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do j=1,ny
               call grad_1d( x, tmpw(:,j,k), ddx(:,j,k), undef=undef )
@@ -2113,7 +2257,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
         do j=1,ny
            do i=1,nx
               call grad_1d( z, tmpu(i,j,:), ddz(i,j,:) )
@@ -2123,7 +2267,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do j=1,ny
               call grad_1d( x, tmpw(:,j,k), ddx(:,j,k) )
@@ -2154,7 +2298,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      !-- scale  undef ꤷƤʤΤ, undef ׻Ϥʤ.
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
      do k=1,nz
         do i=1,nx
            call grad_1d( y, scalex(i,:,k), ddy(i,:,k) )
@@ -2164,7 +2308,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do j=1,ny
         do i=1,nx
            call grad_1d( z, scalex(i,j,:), ddz(i,j,:) )
@@ -2175,7 +2319,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do j=1,ny
               call grad_1d( x, u(:,j,k), ddx(:,j,k), undef=undef )
@@ -2197,7 +2341,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do j=1,ny
               call grad_1d( x, u(:,j,k), ddx(:,j,k) )
@@ -2229,7 +2373,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
         do k=1,nz
            do i=1,nx
               call grad_1d( y, v(i,:,k), ddy(i,:,k), undef=undef )
@@ -2251,7 +2395,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
         do k=1,nz
            do i=1,nx
               call grad_1d( y, v(i,:,k), ddy(i,:,k) )
@@ -2264,7 +2408,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      !-- scale  undef ꤷƤʤΤ, undef ׻Ϥʤ.
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
      do j=1,ny
         do i=1,nx
            call grad_1d( z, scaley(i,j,:), ddz(i,j,:) )
@@ -2274,7 +2418,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 !$omp end parallel
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
      do k=1,nz
         do j=1,ny
            call grad_1d( x, scaley(:,j,k), ddx(:,j,k) )
@@ -2304,7 +2448,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      !-- scale  undef ꤷƤʤΤ, undef ׻Ϥʤ.
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
      do k=1,nz
         do i=1,nx
            call grad_1d( y, scalez(i,:,k), ddy(i,:,k) )
@@ -2315,7 +2459,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
 
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
         do j=1,ny
            do i=1,nx
               call grad_1d( z, w(i,j,:), ddz(i,j,:), undef=undef )
@@ -2337,7 +2481,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
         do j=1,ny
            do i=1,nx
               call grad_1d( z, w(i,j,:), ddz(i,j,:) )
@@ -2349,7 +2493,7 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
      end if
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
      do k=1,nz
         do j=1,ny
            call grad_1d( x, scalez(:,j,k), ddx(:,j,k) )
@@ -2551,7 +2695,7 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
   case ('1')
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do j=1,ny
               call grad_1d( x, phi(:,j,k), val(:,j,k), undef=undef )
@@ -2573,7 +2717,7 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(j,k)
+!$omp do schedule(runtime) private(j,k)
         do k=1,nz
            do j=1,ny
               call grad_1d( x, phi(:,j,k), val(:,j,k) )
@@ -2603,7 +2747,7 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
   case ('2')
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
         do k=1,nz
            do i=1,nx
               call grad_1d( y, phi(i,:,k), val(i,:,k), undef=undef )
@@ -2625,7 +2769,7 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,k)
+!$omp do schedule(runtime) private(i,k)
         do k=1,nz
            do i=1,nx
               call grad_1d( y, phi(i,:,k), val(i,:,k) )
@@ -2655,7 +2799,7 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
   case ('3')
      if(present(undef))then
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
         do j=1,ny
            do i=1,nx
               call grad_1d( z, phi(i,j,:), val(i,j,:), undef=undef )
@@ -2677,7 +2821,7 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
      else
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j)
+!$omp do schedule(runtime) private(i,j)
         do j=1,ny
            do i=1,nx
               call grad_1d( z, phi(i,j,:), val(i,j,:) )
@@ -2723,4 +2867,120 @@ end subroutine
 !-----------------------------------------
 !-----------------------------------------
 
+! ʲ, ꥸʥ zast_2_vel_?d Ǥ뤬, CReSS ǤϿʿ®˴ؤ
+! terrain following ʬѲϤʤΤ, ΤȤʲ롼
+! ȤӤʤ.
+!subroutine zast_2_vel_2d( x, y, zast, u, v, w, uh, vh, wh, undef )
+!! terrain following ɸϤɾƤ®ʿȱľѴ.
+!! , 3 ǥȷϤѴΤǤϤʤ, terrain following ɸϤ
+!! ®ʬǥȷѴ.
+!! , ʿˤϥǥȷϤˤΤбƤ.
+!! ⤷, ɸѴ, vert_coord_conv ⥸塼ѤΤ.
+!  implicit none
+!  real, dimension(:), intent(in) :: x  ! κɸ [m]
+!  real, dimension(:), intent(in) :: y  ! κɸ [m]
+!  real, dimension(size(x),size(y)), intent(in) :: zast  ! terrain Ϥ,  1  [m]
+!  real, dimension(size(x),size(y)), intent(in) :: u  ! zast ˤ [m/s]
+!  real, dimension(size(x),size(y)), intent(in) :: v  ! zast ˤ [m/s]
+!  real, dimension(size(x),size(y)), intent(in) :: w  ! zast ˤľ [m/s]
+!  real, dimension(size(x),size(y)), intent(inout) :: uh  ! ǥȷϤˤ [m/s]
+!  real, dimension(size(x),size(y)), intent(inout) :: vh  ! ǥȷϤˤ [m/s]
+!  real, dimension(size(x),size(y)), intent(inout) :: wh  ! ǥȷϤˤľ [m/s]
+!  real, intent(in), optional :: undef  ! »
+!  integer :: i, j, nx, ny
+!  real, dimension(size(x),size(y)) :: dx, dy
+!  real :: cosx, sinx, cosy, siny
+!
+!  nx=size(x)
+!  ny=size(y)
+!
+!  call grad_2d( x, y, zast, dx, dy )
+!
+!  if(present(undef))then
+!     do j=1,ny
+!        do i=1,nx
+!           if(u(i,j)/=undef.and.v(i,j)/=undef.and.w(i,j)/=undef)then
+!              cosx=1.0/sqrt(1.0+dx(i,j)*dx(i,j))
+!              cosy=1.0/sqrt(1.0+dy(i,j)*dy(i,j))
+!              sinx=dx(i,j)*cosx
+!              siny=dy(i,j)*cosy
+!              uh(i,j)=u(i,j)*cosx
+!              vh(i,j)=v(i,j)*cosy
+!              wh(i,j)=w(i,j)+u(i,j)*sinx+v(i,j)*siny
+!           else
+!              uh(i,j)=undef
+!              vh(i,j)=undef
+!              wh(i,j)=undef
+!           end if
+!        end do
+!     end do
+!  else
+!     do j=1,ny
+!        do i=1,nx
+!           cosx=1.0/sqrt(1.0+dx(i,j)*dx(i,j))
+!           cosy=1.0/sqrt(1.0+dy(i,j)*dy(i,j))
+!           sinx=dx(i,j)*cosx
+!           siny=dy(i,j)*cosy
+!           uh(i,j)=u(i,j)*cosx
+!           vh(i,j)=v(i,j)*cosy
+!           wh(i,j)=w(i,j)+u(i,j)*sinx+v(i,j)*siny
+!        end do
+!     end do
+!  end if
+!
+!end subroutine zast_2_vel_2d
+!
+!!-----------------------------------------
+!!-----------------------------------------
+!
+!subroutine zast_2_vel_3d( x, y, zast, u, v, w, uh, vh, wh, undef )
+!! terrain following ɸϤɾƤ®ʿȱľѴ.
+!! , 3 ǥȷϤѴΤǤϤʤ, terrain following ɸϤ
+!! ®ʬǥȷѴ.
+!! , ʿˤϥǥȷϤˤΤбƤ.
+!! ⤷, ɸѴ, vert_coord_conv ⥸塼ѤΤ.
+!  implicit none
+!  real, dimension(:,:,:), intent(in) :: zast  ! terrain Ϥιٺɸ [m]
+!  real, dimension(size(zast,1)), intent(in) :: x  ! κɸ [m]
+!  real, dimension(size(zast,2)), intent(in) :: y  ! κɸ [m]
+!  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: u  ! zast ˤ [m/s]
+!  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: v  ! zast ˤ [m/s]
+!  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: w  ! zast ˤľ [m/s]
+!  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: uh  ! ǥȷϤˤ [m/s]
+!  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: vh  ! ǥȷϤˤ [m/s]
+!  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: wh  ! ǥȷϤˤľ [m/s]
+!  real, intent(in), optional :: undef  ! »
+!  integer :: k, nx, ny, nz
+!
+!  nx=size(zast,1)
+!  ny=size(zast,2)
+!  nz=size(zast,3)
+!
+!  if(present(undef))then
+!
+!!$omp parallel default(shared)
+!!$omp do schedule(runtime) private(k)
+!     do k=1,nz
+!        call zast_2_vel_2d( x, y, zast(:,:,k), u(:,:,k), v(:,:,k), w(:,:,k),  &
+!  &                         uh(:,:,k), vh(:,:,k), wh(:,:,k), undef )
+!     end do
+!!$omp end do
+!!$omp end parallel
+!
+!  else
+!
+!!$omp parallel default(shared)
+!!$omp do schedule(runtime) private(k)
+!     do k=1,nz
+!        call zast_2_vel_2d( x, y, zast(:,:,k), u(:,:,k), v(:,:,k), w(:,:,k),  &
+!  &                         uh(:,:,k), vh(:,:,k), wh(:,:,k) )
+!     end do
+!!$omp end do
+!!$omp end parallel
+!
+!  end if
+!
+!end subroutine zast_2_vel_3d
+
+
 end module Derivation
