@@ -1405,6 +1405,56 @@ end function
 !-----------------------------------------------------
 !-----------------------------------------------------
 
+real function TTd_2_RH_Bolton( T, Td, dl )
+  ! ٤Ϫ٤м٤׻.
+  ! Bolton μѤƿ˰¿׻. 
+  use Thermo_Const
+  implicit none
+  real, intent(in) :: T  !  [K]
+  real, intent(in) :: Td ! Ϫ [K]
+  integer, intent(in), optional :: dl  ! ǥХå٥
+  real :: e,es
+
+  e=es_Bolton(Td)
+  es=es_Bolton(T)
+
+  TTd_2_RH_Bolton=(e/es)*100.0
+
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Function', 'TTd_2_RH_Bolton', TTd_2_RH_Bolton, '%' )
+  end if
+  
+  return
+end function
+
+!-----------------------------------------------------
+!-----------------------------------------------------
+
+real function TTd_2_RH_tetens( T, Td, dl )
+  ! ٤Ϫ٤м٤׻.
+  ! tetens μѤƿ˰¿׻. 
+  use Thermo_Const
+  implicit none
+  real, intent(in) :: T  !  [K]
+  real, intent(in) :: Td ! Ϫ [K]
+  integer, intent(in), optional :: dl  ! ǥХå٥
+  real :: e,es
+
+  e=tetens(Td)
+  es=tetens(T)
+
+  TTd_2_RH_tetens=(e/es)*100.0
+
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Function', 'TTd_2_RH_tetens', TTd_2_RH_tetens, '%' )
+  end if
+  
+  return
+end function
+
+!-----------------------------------------------------
+!-----------------------------------------------------
+
 real function rho_ocean( pres, temp, sal, dl )
 ! UNESCO (1981) ξѤ, ̩٤׻.
 ! Ǥη׻ˤϰʲΤ褦ʥɺνޤޤƤ뤳Ȥդ.
@@ -1546,5 +1596,4 @@ end function rho_ocean
 
 
 
-
 end module
